using System;
using System.Collections.Generic;

class Program : System.Windows.Forms.Form
{
	const int	WIDTH = 240;		//	仮想画面の幅
	const int	HEIGHT = 180;		//	仮想画面の高さ
	int			mScore;				//	得点
	int			mWY = HEIGHT / 2;	//	壁穴のY座標
	float		mDY;				//	Y座標の移動差分
	float		mKey;				//	キー押下状態
	float		mY = HEIGHT / 2;	//	プレイヤーのY座標
	bool		mGameOver;			//	ゲームオーバーかどうか
	System.Drawing.RectangleF[]	mWall = new System.Drawing.RectangleF[ 8 ];	//	壁座標

	Program()
	{
		SetStyle( System.Windows.Forms.ControlStyles.Opaque, true );	//	ちらつき防止
		TUGames.GX.Init( this, WIDTH, HEIGHT, 3 );	//	自身、ウィンドウサイズ横、縦を指定してライブラリ初期化

		for( int i = 0, x = 100; i < 8; i += 2, x += 60 ){	//	壁を８つ生成
			setWall( i, x );	//	壁を設置
		}

		TUGames.PF.StartLoop( 30,	//	fpsを指定してループを開始し、更新処理と描画処理を登録
		() =>{	//	更新処理
			if( mGameOver ){	//	ゲームオーバーの場合
				return;
			}

			mScore++;			//	得点加算
			mDY += mKey;		//	キー入力状態に応じてY方向移動量変更
			mY += mDY;			//	プレイヤーY座標変更
			if( mY < 0 || mY > HEIGHT ){	//	プレイヤーが画面の上下範囲外に達した場合
				mGameOver = true;			//	ゲームオーバー
			}
			for( int i = 0; i < mWall.Length; i++ ){		//	全ての壁
				mWall[ i ].X -= mScore / 256.0f + 0.25f;	//	壁スクロール
				if( ( i & 1 ) == 1 && mWall[ i ].X < -20 ){	//	下側の壁が画面左端を超えた場合
					setWall( i - 1, WIDTH );				//	新たに壁を設置
				}
				if( mWall[ i ].Contains( 45, mY ) ){		//	プレイヤーの中央座標と壁が重なった場合
					mGameOver = true;						//	ゲームオーバー
				}
			}
		},
		() =>{	//	描画処理
			TUGames.GX.Clear();	//	設定された背景色で画面初期化
			TUGames.GX.FillRect( 0, 0, WIDTH, HEIGHT, 0xffff0000 );	//	背景の矩形を描画
			TUGames.GX.FillRect( 40, mY - 5, 10, 10, 0xff00ffff );	//	プレイヤーの矩形を描画
			TUGames.GX.FillRects( mWall, 0xff00ff00 );				//	全ての壁の矩形を描画
			TUGames.GX.DrawString( "SCORE " + mScore, 0, 0, 0, 0xffffffff );	//	得点文字列を描画
			if( mGameOver ){	//	ゲームオーバーの場合
				TUGames.GX.DrawString( "GAME OVER", WIDTH / 2 - 18, HEIGHT / 2 - 2, 0, 0xffffffff );	//	ゲームオーバー文字列を描画
			}
			TUGames.GX.SwapBuffers();	//	表裏画面を切り替え、描画内容を表示
		} );

	}

	//	キーが押された場合のイベント
	protected override void OnKeyDown( System.Windows.Forms.KeyEventArgs e )
	{
		mKey = -0.25f;		//	キー押下状態
	}

	//	キーが離された場合のイベント
	protected override void OnKeyUp( System.Windows.Forms.KeyEventArgs e )
	{
		mKey = 0.25f;		//	キー押下状態
	}

	//	壁を設置
	void setWall( int idx, int x )
	{
		mWY += TUGames.UT.Rnd( 41 ) - 20;	//	壁穴Y座標を前回位置から+-20の範囲で変更
		mWY = Math.Min( mWY, HEIGHT - 40 );	//	壁穴Y座標上限チェック
		mWY = Math.Max( mWY, 40 );			//	壁穴Y座標下限チェック
		mWall[ idx++ ] = new System.Drawing.RectangleF( x, 0       , 20, mWY - 20 );	//	壁上部生成
		mWall[ idx   ] = new System.Drawing.RectangleF( x, mWY + 20, 20, 160 - mWY );	//	壁下部生成
	}

	static void Main( string[] args )
	{
		System.Windows.Forms.Application.Run( new Program() );	//	このアプリを実行
	}
}
