using System;
using System.Collections.Generic;

class Program : TUGames.Client
{
	const short		WIDTH = 960;		//	仮想画面の幅
	const short		HEIGHT = 720;		//	仮想画面の高さ
	int				mScore;				//	得点
	int				mWY = HEIGHT >> 1;	//	壁穴のY座標
	int				mDY;				//	Y座標の移動差分
	int				mKey;				//	キー押下状態
	int				mY = HEIGHT >> 1;	//	プレイヤーのY座標
	bool			mGameOver;			//	ゲームオーバーかどうか
	System.Drawing.Rectangle[]	mWall = new System.Drawing.Rectangle[ 8 ];	//	壁座標

	Program()
	{
		SetStyle( System.Windows.Forms.ControlStyles.Opaque, true );	//	ちらつき防止
		cmd( GX_Init( WIDTH, HEIGHT, 0.75f, 0 ) );

		cmd( TF_Create() );
		cmd( new byte[]{	MOV_AR,	0x00,	DX,	} );
		cmd( new byte[]{	MOV_RA,	BX,	0x00	}	);
		cmd( PS_CreateBX( "SCORE" , 0     , 0, 0xffff, 0xffaa, 0xc000 ) );
		cmd( new byte[]{	MOV_AR,	0x02,	DX,	} );

		cmd( PS_CreateBX( "01234" , 32 * 5, 0, 0xffff, 0xffaa, 0xc000 ) );
		cmd( new byte[]{	MOV_AR,	0x04,	DX,	} );

		cmd( PS_CreateBX( "GAME OVER" , 32 * 10, 32 * 10, 0xffff, 0xffaa, 0xc000 ) );
		cmd( new byte[]{	MOV_AR,	0x06,	DX,	} );

		for( int i = 0, x = 400; i < 8; i += 2, x += 240 ){	//	壁を８つ生成
			setWall( i, x );	//	壁を設置
		}

		var	cm = new List<byte>();
		TUGames.PF.StartLoop( 30,	//	fpsを指定してループを開始し、更新処理と描画処理を登録
		() =>{	//	更新処理
			if( mGameOver ){	//	ゲームオーバーの場合
				return;
			}

			mScore++;			//	得点加算
			mDY += mKey;		//	キー入力状態に応じてY方向移動量変更
			mY += mDY;			//	プレイヤーY座標変更
			if( mY < 0 || mY > HEIGHT ){	//	プレイヤーが画面の上下範囲外に達した場合
				mGameOver = true;			//	ゲームオーバー
			}
			for( int i = 0; i < mWall.Length; i++ ){		//	全ての壁
				mWall[ i ].X -= ( mScore >> 6 ) + 1;			//	壁スクロール
				if( ( i & 1 ) == 1 && mWall[ i ].X < -80 ){	//	下側の壁が画面左端を超えた場合
					setWall( i - 1, WIDTH );				//	新たに壁を設置
				}
				if( mWall[ i ].Contains( 180, mY ) ){		//	プレイヤーの中央座標と壁が重なった場合
					mGameOver = true;						//	ゲームオーバー
				}
			}
		},
		() =>{	//	描画処理
			cm.Clear();
			cm.AddRange( GX_Clear() );	//	設定された背景色で画面初期化
			cm.AddRange( GX_FillRect( 0, 0, (short)WIDTH, (short)HEIGHT, 0xf0 ) );	//	背景の矩形を描画
			cm.AddRange( GX_FillRect( 160, (short)( mY - 20 ), 40, 40, 0xcf ) );	//	プレイヤーの矩形を描画

			//	全ての壁の矩形を描画
			for( int i = 0; i < mWall.Length; i++ ){
				var	w = mWall[ i ];
				cm.AddRange( GX_FillRect( (short)w.X, (short)w.Y, (short)w.Width, (short)w.Height, 0xcc ) );
			}

			cm.AddRange( new byte[]{	MOV_RA,	BX,	0x04	}	);
			cm.AddRange( PS_setStringBX( ( "" + mScore ).PadLeft( 5, ' ' ) ) );

			cm.AddRange( new byte[]{	MOV,	AH,	PS_DRW	}	);
			if( mGameOver ){	//	ゲームオーバーの場合
				cm.AddRange( new byte[]{
					MOV_RA,	BX,	6,
					INT,	PS
				} );
			}
			for( int i = 1; i < 3; i++ ){
				cm.AddRange( new byte[]{
					MOV_RA,	BX,	(byte)( i << 1 ),
					INT,	PS,
				} );
			}

			cm.AddRange( GX_SwapBuffers() );	//	表裏画面を切り替え、描画内容を表示

			cmd( cm.ToArray() );
		} );

	}

	//	キーが押された場合のイベント
	protected override void OnKeyDown( System.Windows.Forms.KeyEventArgs e )
	{
		mKey = -1;		//	キー押下状態
	}

	//	キーが離された場合のイベント
	protected override void OnKeyUp( System.Windows.Forms.KeyEventArgs e )
	{
		mKey = 1;		//	キー押下状態
	}

	//	壁を設置
	void setWall( int idx, int x )
	{
		mWY += TUGames.UT.Rnd( 161 ) - 80;		//	壁穴Y座標を前回位置から+-80の範囲で変更
		mWY = Math.Min( mWY, HEIGHT - 160 );	//	壁穴Y座標上限チェック
		mWY = Math.Max( mWY, 160 );				//	壁穴Y座標下限チェック
		mWall[ idx++ ] = new System.Drawing.Rectangle( x, 0       , 80, mWY - 80 );	//	壁上部生成
		mWall[ idx   ] = new System.Drawing.Rectangle( x, mWY + 80, 80, 640 - mWY );	//	壁下部生成
	}

	static void Main( string[] args )
	{
		System.Windows.Forms.Application.Run( new Program() );	//	このアプリを実行
	}
}
