using System;
using System.Collections.Generic;

class Program : System.Windows.Forms.Form
{
	static readonly int TILE_SIZE = 32;
	static readonly int MAP_WIDTH = 19;
	static readonly int MAP_HEIGHT = 15;

	List<int>	mX, mY;
	int			mAX, mAY, mDX, mDY, mFoodX, mFoodY;
	Random		mRnd = new Random();
	bool		mGameOver, mGameClear;

	Program()
	{
		SetStyle( System.Windows.Forms.ControlStyles.Opaque, true );	//	ちらつき防止
		TUGames.GX.Init( this, TILE_SIZE * MAP_WIDTH, TILE_SIZE * MAP_HEIGHT );	//	自身、ウィンドウサイズ横、縦を指定してライブラリ初期化

		start();
		TUGames.PF.StartLoop( 10, update, draw );	//	fpsを指定してループを開始し、更新処理と描画処理を登録
	}

	void addFood()
	{
		int		x = mFoodX = mRnd.Next( MAP_WIDTH  + 1 >> 1 ) << 1;
		int		y = mFoodY = mRnd.Next( MAP_HEIGHT + 1 >> 1 ) << 1;

		while( isBody( mFoodX, mFoodY ) ){
			mFoodX += 2;

			if( x == mFoodX && y == mFoodY ){
				mGameClear = true;
				break;
			}

			if( mFoodX < MAP_WIDTH ){
				continue;
			}
			mFoodX = 0;
			mFoodY += 2;
			if( mFoodY >= MAP_HEIGHT ){
				mFoodY = 0;
			}
		}

		mX.Add( mX[ mX.Count - 1 ] );
		mY.Add( mY[ mY.Count - 1 ] );
	}

	void draw()
	{
		TUGames.GX.Clear();	//	設定された背景色で画面初期化

		TUGames.GX.FillRectZ( mFoodX * TILE_SIZE, mFoodY * TILE_SIZE, 0, TILE_SIZE, TILE_SIZE, 0xff00ffff );	//	座標XYZへ、幅高さWH、ABGR形式で指定した色で矩形を描画
		for( int i = mX.Count - 1; i >= 0; i-- ){
			TUGames.GX.FillRectZ( mX[ i ] * TILE_SIZE, mY[ i ] * TILE_SIZE, 0, TILE_SIZE, TILE_SIZE, 0xffffff00 );	//	座標XYZへ、幅高さWH、ABGR形式で指定した色で矩形を描画
		}

		if( mGameOver ){
			TUGames.GX.DrawString( "ＧＡＭＥ　ＯＶＥＲ", 224, 216, 1, 0xffffffff );	//	座標XYZへ、ABGR形式で指定した色で文字列を描画
		}
		if( mGameClear ){
			TUGames.GX.DrawString( "ＧＡＭＥ　ＣＬＥＡＲ", 224, 216, 1, 0xffffffff );	//	座標XYZへ、ABGR形式で指定した色で文字列を描画
		}
		if( mGameOver || mGameClear ){
			TUGames.GX.DrawString( "ＳＣＯＲＥ　" + mX.Count, 224, 264, 1, 0xffffffff );	//	座標XYZへ、ABGR形式で指定した色で文字列を描画
		}

		TUGames.GX.SwapBuffers();	//	表裏画面を切り替え、描画内容を表示
	}

	bool isBody( int x, int y )
	{
		for( int i = mX.Count - 1; i >= 0; i-- ){
			if( mX[ i ] == x && mY[ i ] == y ){
				return( true );
			}
		}
		return( false );
	}

	void start()
	{
		mGameClear = false;
		mGameOver = false;
		mX = new List<int>();
		mY = new List<int>();
		mX.Add( MAP_WIDTH  >> 2 << 1 );
		mY.Add( MAP_HEIGHT >> 2 << 1 );
		mDX = 1;
		mDY = 0;
		addFood();
		addFood();
		addFood();
	}

	void update()
	{
		if( mGameOver || mGameClear ){
			if( !TUGames.PF.IsKeyDown( System.Windows.Forms.Keys.Enter ) ){
				return;
			}
			start();
		}

		if( TUGames.PF.IsKeyDown( System.Windows.Forms.Keys.Left  ) ){	mDX = -1;	mDY =  0;	}	//	引数で指定されたキーが押されている場合
		if( TUGames.PF.IsKeyDown( System.Windows.Forms.Keys.Right ) ){	mDX =  1;	mDY =  0;	}	//	引数で指定されたキーが押されている場合
		if( TUGames.PF.IsKeyDown( System.Windows.Forms.Keys.Up    ) ){	mDX =  0;	mDY = -1;	}	//	引数で指定されたキーが押されている場合
		if( TUGames.PF.IsKeyDown( System.Windows.Forms.Keys.Down  ) ){	mDX =  0;	mDY =  1;	}	//	引数で指定されたキーが押されている場合

		if( ( mX[ 0 ] & 1 ) == 0 && ( mY[ 0 ] & 1 ) == 0 ){
			mAX = mDX;
			mAY = mDY;
		}

		int		x = mX[ 0 ] + mAX;
		int		y = mY[ 0 ] + mAY;
		if( isBody( x, y ) || x < 0 || y < 0 || x >= MAP_WIDTH || y >= MAP_HEIGHT ){
			mGameOver = true;
		}

		mX.Insert( 0, x );
		mY.Insert( 0, y );
		mX.RemoveAt( mX.Count - 1 );
		mY.RemoveAt( mY.Count - 1 );

		if( x == mFoodX && y == mFoodY ){
			addFood();
		}
	}

	static void Main( string[] args )
	{
		System.Windows.Forms.Application.Run( new Program() );
	}
}
