using System;
using System.Collections.Generic;

class Program : System.Windows.Forms.Form
{
	static readonly int	TILE_SIZE  = 12;
	static readonly int	MAP_WIDTH  = 14;
	static readonly int	MAP_HEIGHT = 25;
	static readonly int	SCR_WIDTH  = MAP_WIDTH * TILE_SIZE;
	static readonly int	SCR_HEIGHT = MAP_HEIGHT * TILE_SIZE;
	static readonly int	SCALE = 2;
	static readonly int	WND_WIDTH  = SCR_WIDTH * SCALE;
	static readonly int	WND_HEIGHT = SCR_HEIGHT * SCALE;
	static readonly int WAIT = 60;

	static readonly byte[,,] mBlock = {
		{
			{ 0, 0, 0, 0 },
			{ 1, 1, 1, 1 },
			{ 0, 0, 0, 0 },
			{ 0, 0, 0, 0 },
		},{
			{ 0, 0, 0, 0 },
			{ 0, 1, 1, 1 },
			{ 0, 1, 0, 0 },
			{ 0, 0, 0, 0 },
		},{
			{ 0, 0, 0, 0 },
			{ 0, 1, 1, 0 },
			{ 0, 1, 1, 0 },
			{ 0, 0, 0, 0 },
		},{
			{ 0, 0, 0, 0 },
			{ 1, 1, 0, 0 },
			{ 0, 1, 1, 0 },
			{ 0, 0, 0, 0 },
		},{
			{ 0, 0, 0, 0 },
			{ 1, 1, 1, 0 },
			{ 0, 1, 0, 0 },
			{ 0, 0, 0, 0 },
		},{
			{ 0, 0, 0, 0 },
			{ 1, 1, 1, 0 },
			{ 0, 0, 1, 0 },
			{ 0, 0, 0, 0 },
		},{
			{ 0, 0, 0, 0 },
			{ 0, 1, 1, 0 },
			{ 1, 1, 0, 0 },
			{ 0, 0, 0, 0 },
		}
	};

	byte[,]		mField = {
		{ 9, 9, 9, 9, 9, 7, 7, 7, 7, 9, 9, 9, 9, 9 },
		{ 9, 9, 9, 9, 9, 7, 7, 7, 7, 9, 9, 9, 9, 9 },
		{ 9, 8, 8, 8, 8, 7, 7, 7, 7, 8, 8, 8, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 9 },
		{ 9, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 9 },
		{ 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9 },
	};

	uint[]	mTile;
	int		mX, mY, mA, mWait;
	byte	mT, mNext;
	Random	mRnd = new Random();
	bool	mGameOver;

	Program()
	{
		SetStyle( System.Windows.Forms.ControlStyles.Opaque, true );	//	ちらつき防止
		TUGames.GX.Init( this, WND_WIDTH, WND_HEIGHT );	//	自身、ウィンドウサイズ横、縦を指定してライブラリ初期化

		mTile = TUGames.GX.LoadTextureDiv( "../assets/tile.png", TILE_SIZE, TILE_SIZE );	//	指定されたファイルをテクスチャ画像として分割して読み込む
		mNext = (byte)mRnd.Next( 7 );
		next();
		start();
		TUGames.PF.StartLoop( 60, update, draw );	//	fpsを指定してループを開始し、更新処理と描画処理を登録
	}

	void draw()
	{
		TUGames.GX.Clear();	//	設定された背景色で画面初期化

		for( int y = 0; y < MAP_HEIGHT; y++ ){
			for( int x = 0; x < MAP_WIDTH; x++ ){
				TUGames.GX.DrawTexture( mTile[ mField[ y, x ] ],
				                        x * TILE_SIZE * SCALE, y * TILE_SIZE * SCALE, 0,
				                        TILE_SIZE * SCALE, TILE_SIZE * SCALE );
			}
		}

		TUGames.GX.SwapBuffers();	//	表裏画面を切り替え、描画内容を表示
	}

	void next()
	{
		mX = 5;
		mY = 2;
		mT = mNext;
		mWait = WAIT;

		mA = 0;
		if( TUGames.PF.GetKey( System.Windows.Forms.Keys.X ) > 0 )	mA = 3;
		if( TUGames.PF.GetKey( System.Windows.Forms.Keys.Z ) > 0 )	mA = 1;

		if( !put( mX, mY, mT, mA, true, false ) ){
			mGameOver = true;
		}

		put( 5, -1, mNext, 0, false, false );
		mNext = (byte)mRnd.Next( 7 );
		put( 5, -1, mNext, 0, true, false );
	}

	void start()
	{
	}

	bool put( int x, int y, byte t, int a, bool s, bool test )
	{
		for( int j = 0; j < 4; j++ ){
			for( int i = 0; i < 4; i++ ){
				int[]	p = { i, 3 - j, 3 - i,     j };
				int[]	q = { j,     i, 3 - j, 3 - i };
				if( mBlock[ t, q[ a ], p[ a ] ] == 0 ){
					continue;
				}

				byte	v = t;
				if( !s ){
					v = 7;
				}else if( mField[ y + j, x + i ] != 7 ){
					return( false );
				}

				if( !test ){
					mField[ y + j, x + i ] = v;
				}
			}
		}

		return( true );
	}


	void update()
	{
		if( mGameOver ){
			return;
		}

		if( mWait <= WAIT / 2 ){
			wait();
			return;
		}

		put( mX, mY, mT, mA, false, false );

		int		a = mA;
		if( TUGames.PF.GetKey( System.Windows.Forms.Keys.X ) == 2 )	a--;
		if( TUGames.PF.GetKey( System.Windows.Forms.Keys.Z ) == 2 )	a++;
		a &= 3;
		if( put( mX, mY, mT, a, true, true ) ){
			mA = a;
		}

		int		x = mX;
		if( TUGames.PF.GetKey( System.Windows.Forms.Keys.Left  ) == 2 || TUGames.PF.GetKey( System.Windows.Forms.Keys.Left  ) > 20 )	x--;
		if( TUGames.PF.GetKey( System.Windows.Forms.Keys.Right ) == 2 || TUGames.PF.GetKey( System.Windows.Forms.Keys.Right ) > 20 )	x++;
		if( put( x, mY, mT, mA, true, true ) ){
			mX = x;
		}

		if( put( mX, mY + 1, mT, mA, true, true ) ){
			mY++;
			mWait = WAIT;
		}else{
			mWait--;
		}

		put( mX, mY, mT, mA, true, false );
	}

	void wait()
	{
		if( --mWait == 0 ){
			next();
		}

		if( mWait == WAIT / 2 - 1 ){
			for( int y = 22; y > 2; y-- ){
				int		n = 0;
				for( int x = 2; x < 12; x++ ){
					if( mField[ y, x ] < 7 ){
						n++;
					}
				}
				if( n != 10 ){
					continue;
				}
				for( int x = 2; x < 12; x++ ){
					mField[ y, x ] = 10;
				}
			}
		}

		if( mWait == 1 ){
			for( int y = 22; y > 2; y-- ){
				if( mField[ y, 2 ] != 10 ){
					continue;
				}
				mWait = WAIT / 2 - 2;
				for( int i = y; i > 3; i-- ){
					for( int x = 2; x < 12; x++ ){
						mField[ i, x ] = mField[ i - 1, x ];
					}
				}
				for( int x = 2; x < 12; x++ ){
					mField[ 3, x ] = 7;
				}
				y++;
			}
		}
	}

	static void Main( string[] args )
	{
		System.Windows.Forms.Application.Run( new Program() );
	}
}
